@@ -29,6 +29,7 @@ Find all information about changes of the different versions of this module
 
 from __future__ import (absolute_import, print_function, unicode_literals)
 import xml.dom.minidom
+import xml.parsers.expat
 import time
 import sys
 import urllib
@@ -84,6 +85,12 @@ class OsmTypeAlreadyExistsError(OsmApiError):
     pass
 
 
+class XmlResponseInvalidError(OsmApiError):
+    """
+    Error if the XML response from the OpenStreetMap API is invalid
+    """
+
+
 class ApiError(OsmApiError):
     """
     Error class, is thrown when an API request fails
@@ -122,6 +129,20 @@ class NotSubscribedApiError(ApiError):
     pass
 
 
+class ElementDeletedApiError(ApiError):
+    """
+    Error when the requested element is deleted
+    """
+    pass
+
+
+class ResponseEmptyApiError(ApiError):
+    """
+    Error when the response to the request is empty
+    """
+    pass
+
+
 class OsmApi:
     """
     Main class of osmapi, instanciate this class to use osmapi
@@ -228,8 +249,12 @@ class OsmApi:
         self._session = self._get_http_session()
 
     def __del__(self):
-        if self._changesetauto:
-            self._changesetautoflush(True)
+        try:
+            if self._changesetauto:
+                self._changesetautoflush(True)
+        except ResponseEmptyApiError:
+            pass
+
         return None
 
     ##################################################
@@ -273,9 +298,8 @@ class OsmApi:
         """
         uri = "/api/capabilities"
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("api")[0]
+
+        data = self._OsmResponseToDom(data, tag="api", single=True)
         result = {}
         for elem in data.childNodes:
             if elem.nodeType != elem.ELEMENT_NODE:
@@ -312,16 +336,15 @@ class OsmApi:
 
         If `NodeVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         uri = "/api/0.6/node/%s" % (NodeId)
         if NodeVersion != -1:
             uri += "/%s" % (NodeVersion)
         data = self._get(uri)
-        if not data:
-            return data
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("node")[0]
+        data = self._OsmResponseToDom(data, tag="node", single=True)
         return self._DomParseNode(data)
 
     def NodeCreate(self, NodeData):
@@ -436,6 +459,9 @@ class OsmApi:
 
         If there is already an open changeset,
         `OsmApi.ChangesetAlreadyOpenError` is raised.
+
+        If the requested element has already been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         return self._do("delete", "node", NodeData)
 
@@ -454,11 +480,10 @@ class OsmApi:
         """
         uri = "/api/0.6/node/%s/history" % NodeId
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        nodes = self._OsmResponseToDom(data, tag="node")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("node"):
-            data = self._DomParseNode(data)
+        for node in nodes:
+            data = self._DomParseNode(node)
             result[data["version"]] = data
         return result
 
@@ -487,11 +512,10 @@ class OsmApi:
         """
         uri = "/api/0.6/node/%d/ways" % NodeId
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        ways = self._OsmResponseToDom(data, tag="way")
         result = []
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("way"):
-            data = self._DomParseWay(data)
+        for way in ways:
+            data = self._DomParseWay(way)
             result.append(data)
         return result
 
@@ -529,11 +553,10 @@ class OsmApi:
         """
         uri = "/api/0.6/node/%d/relations" % NodeId
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        relations = self._OsmResponseToDom(data, tag="relation")
         result = []
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("relation"):
-            data = self._DomParseRelation(data)
+        for relation in relations:
+            data = self._DomParseRelation(relation)
             result.append(data)
         return result
 
@@ -555,11 +578,10 @@ class OsmApi:
         node_list = ",".join([str(x) for x in NodeIdList])
         uri = "/api/0.6/nodes?nodes=%s" % node_list
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        nodes = self._OsmResponseToDom(data, tag="node")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("node"):
-            data = self._DomParseNode(data)
+        for node in nodes:
+            data = self._DomParseNode(node)
             result[data["id"]] = data
         return result
 
@@ -586,17 +608,16 @@ class OsmApi:
 
         If `WayVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         uri = "/api/0.6/way/%s" % (WayId)
         if WayVersion != -1:
             uri += "/%s" % (WayVersion)
         data = self._get(uri)
-        if not data:
-            return data
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("way")[0]
-        return self._DomParseWay(data)
+        way = self._OsmResponseToDom(data, tag="way", single=True)
+        return self._DomParseWay(way)
 
     def WayCreate(self, WayData):
         """
@@ -707,6 +728,9 @@ class OsmApi:
 
         If there is already an open changeset,
         `OsmApi.ChangesetAlreadyOpenError` is raised.
+
+        If the requested element has already been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         return self._do("delete", "way", WayData)
 
@@ -725,11 +749,10 @@ class OsmApi:
         """
         uri = "/api/0.6/way/%s/history" % (WayId)
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        ways = self._OsmResponseToDom(data, tag="way")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("way"):
-            data = self._DomParseWay(data)
+        for way in ways:
+            data = self._DomParseWay(way)
             result[data["version"]] = data
         return result
 
@@ -767,11 +790,10 @@ class OsmApi:
         """
         uri = "/api/0.6/way/%d/relations" % WayId
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        relations = self._OsmResponseToDom(data, tag="relation")
         result = []
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("relation"):
-            data = self._DomParseRelation(data)
+        for relation in relations:
+            data = self._DomParseRelation(relation)
             result.append(data)
         return result
 
@@ -789,6 +811,9 @@ class OsmApi:
             ]
 
         The `WayId` is a unique identifier for a way.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         uri = "/api/0.6/way/%s/full" % (WayId)
         data = self._get(uri)
@@ -811,11 +836,10 @@ class OsmApi:
         way_list = ",".join([str(x) for x in WayIdList])
         uri = "/api/0.6/ways?ways=%s" % way_list
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        ways = self._OsmResponseToDom(data, tag="way")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("way"):
-            data = self._DomParseWay(data)
+        for way in ways:
+            data = self._DomParseWay(way)
             result[data["id"]] = data
         return result
 
@@ -851,17 +875,16 @@ class OsmApi:
 
         If `RelationVersion` is supplied, this specific version is returned,
         otherwise the latest version is returned.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         uri = "/api/0.6/relation/%s" % (RelationId)
         if RelationVersion != -1:
             uri += "/%s" % (RelationVersion)
         data = self._get(uri)
-        if not data:
-            return data
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("relation")[0]
-        return self._DomParseRelation(data)
+        relation = self._OsmResponseToDom(data, tag="relation", single=True)
+        return self._DomParseRelation(relation)
 
     def RelationCreate(self, RelationData):
         """
@@ -999,6 +1022,9 @@ class OsmApi:
 
         If there is already an open changeset,
         `OsmApi.ChangesetAlreadyOpenError` is raised.
+
+        If the requested element has already been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         return self._do("delete", "relation", RelationData)
 
@@ -1017,11 +1043,10 @@ class OsmApi:
         """
         uri = "/api/0.6/relation/%s/history" % (RelationId)
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        relations = self._OsmResponseToDom(data, tag="relation")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("relation"):
-            data = self._DomParseRelation(data)
+        for relation in relations:
+            data = self._DomParseRelation(relation)
             result[data["version"]] = data
         return result
 
@@ -1060,11 +1085,10 @@ class OsmApi:
         """
         uri = "/api/0.6/relation/%d/relations" % RelationId
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        relations = self._OsmResponseToDom(data, tag="relation")
         result = []
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("relation"):
-            data = self._DomParseRelation(data)
+        for relation in relations:
+            data = self._DomParseRelation(relation)
             result.append(data)
         return result
 
@@ -1088,6 +1112,9 @@ class OsmApi:
 
         If you don't need all levels, use `OsmApi.RelationFull`
         instead, which return only 2 levels.
+
+        If any relation (on any level) has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         data = []
         todo = [RelationId]
@@ -1122,6 +1149,9 @@ class OsmApi:
         The `RelationId` is a unique identifier for a way.
 
         If you need all levels, use `OsmApi.RelationFullRecur`.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         uri = "/api/0.6/relation/%s/full" % (RelationId)
         data = self._get(uri)
@@ -1145,11 +1175,10 @@ class OsmApi:
         relation_list = ",".join([str(x) for x in RelationIdList])
         uri = "/api/0.6/relations?relations=%s" % relation_list
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
+        relations = self._OsmResponseToDom(data, tag="relation")
         result = {}
-        osm_data = data.getElementsByTagName("osm")[0]
-        for data in osm_data.getElementsByTagName("relation"):
-            data = self._DomParseRelation(data)
+        for relation in relations:
+            data = self._DomParseRelation(relation)
             result[data["id"]] = data
         return result
 
@@ -1187,10 +1216,8 @@ class OsmApi:
         if (include_discussion):
             path += "?include_discussion=true"
         data = self._get(path)
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("changeset")[0]
-        return self._DomParseChangeset(data)
+        changeset = self._OsmResponseToDom(data, tag="changeset", single=True)
+        return self._DomParseChangeset(changeset)
 
     def ChangesetUpdate(self, ChangesetTags={}):
         """
@@ -1208,7 +1235,8 @@ class OsmApi:
             ChangesetTags["created_by"] = self._created_by
         self._put(
             "/api/0.6/changeset/%s" % (self._CurrentChangesetId),
-            self._XmlBuild("changeset", {"tag": ChangesetTags})
+            self._XmlBuild("changeset", {"tag": ChangesetTags}),
+            return_value=False
         )
         return self._CurrentChangesetId
 
@@ -1253,7 +1281,8 @@ class OsmApi:
             raise NoChangesetOpenError("No changeset currently opened")
         self._put(
             "/api/0.6/changeset/%s/close" % (self._CurrentChangesetId),
-            ""
+            "",
+            return_value=False
         )
         CurrentChangesetId = self._CurrentChangesetId
         self._CurrentChangesetId = 0
@@ -1293,9 +1322,15 @@ class OsmApi:
             "/api/0.6/changeset/%s/upload" % (self._CurrentChangesetId),
             data.encode("utf-8")
         )
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("diffResult")[0]
-        data = [x for x in data.childNodes if x.nodeType == x.ELEMENT_NODE]
+        try:
+            data = xml.dom.minidom.parseString(data)
+            data = data.getElementsByTagName("diffResult")[0]
+            data = [x for x in data.childNodes if x.nodeType == x.ELEMENT_NODE]
+        except (xml.parsers.expat.ExpatError, IndexError) as e:
+            raise XmlResponseInvalidError(
+                "The XML response from the OSM API is invalid: %r" % e
+            )
+
         for i in range(len(ChangesData)):
             if ChangesData[i]["action"] == "delete":
                 ChangesData[i]["data"].pop("version")
@@ -1379,11 +1414,9 @@ class OsmApi:
             uri += "?" + urllib.urlencode(params)
 
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("changeset")
+        changesets = self._OsmResponseToDom(data, tag="changeset")
         result = {}
-        for curChangeset in data:
+        for curChangeset in changesets:
             tmpCS = self._DomParseChangeset(curChangeset)
             result[tmpCS["id"]] = tmpCS
         return result
@@ -1421,10 +1454,8 @@ class OsmApi:
             "/api/0.6/changeset/%s/comment" % (ChangesetId),
             params
         )
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("changeset")[0]
-        return self._DomParseChangeset(data)
+        changeset = self._OsmResponseToDom(data, tag="changeset", single=True)
+        return self._DomParseChangeset(changeset)
 
     def ChangesetSubscribe(self, ChangesetId):
         """
@@ -1463,10 +1494,8 @@ class OsmApi:
                 raise AlreadySubscribedApiError(e.status, e.reason, e.payload)
             else:
                 raise
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("changeset")[0]
-        return self._DomParseChangeset(data)
+        changeset = self._OsmResponseToDom(data, tag="changeset", single=True)
+        return self._DomParseChangeset(changeset)
 
     def ChangesetUnsubscribe(self, ChangesetId):
         """
@@ -1505,10 +1534,8 @@ class OsmApi:
                 raise NotSubscribedApiError(e.status, e.reason, e.payload)
             else:
                 raise
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
-        data = data.getElementsByTagName("changeset")[0]
-        return self._DomParseChangeset(data)
+        changeset = self._OsmResponseToDom(data, tag="changeset", single=True)
+        return self._DomParseChangeset(changeset)
 
     ##################################################
     # Notes                                          #
@@ -1576,12 +1603,8 @@ class OsmApi:
         """
         uri = "/api/0.6/notes/%s" % (id)
         data = self._get(uri)
-        data = xml.dom.minidom.parseString(data)
-        osm_data = data.getElementsByTagName("osm")[0]
-
-        noteElement = osm_data.getElementsByTagName("note")[0]
-        note = self._DomParseNote(noteElement)
-        return note
+        noteElement = self._OsmResponseToDom(data, tag="note", single=True)
+        return self._DomParseNote(noteElement)
 
     def NoteCreate(self, NoteData):
         """
@@ -1622,6 +1645,9 @@ class OsmApi:
 
         If no authentication information are provided,
         `OsmApi.UsernamePasswordMissingError` is raised.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
         """
         path = "/api/0.6/notes/%s/reopen" % NoteId
         return self._NoteAction(path, comment, optionalAuth=False)
@@ -1661,13 +1687,8 @@ class OsmApi:
         result = self._post(uri, None, optionalAuth=optionalAuth)
 
         # parse the result
-        data = xml.dom.minidom.parseString(result)
-        osm_data = data.getElementsByTagName("osm")[0]
-
-        noteElement = osm_data.getElementsByTagName("note")[0]
-        note = self._DomParseNote(noteElement)
-
-        return note
+        noteElement = self._OsmResponseToDom(result, tag="note", single=True)
+        return self._DomParseNote(noteElement)
 
     ##################################################
     # Other                                          #
@@ -1708,8 +1729,14 @@ class OsmApi:
                 data: {}
             }
         """
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osm")[0]
+        try:
+            data = xml.dom.minidom.parseString(data)
+            data = data.getElementsByTagName("osm")[0]
+        except (xml.parsers.expat.ExpatError, IndexError) as e:
+            raise XmlResponseInvalidError(
+                "The XML response from the OSM API is invalid: %r" % e
+            )
+
         result = []
         for elem in data.childNodes:
             if elem.nodeName == "node":
@@ -1742,8 +1769,14 @@ class OsmApi:
                 data: {}
             }
         """
-        data = xml.dom.minidom.parseString(data)
-        data = data.getElementsByTagName("osmChange")[0]
+        try:
+            data = xml.dom.minidom.parseString(data)
+            data = data.getElementsByTagName("osmChange")[0]
+        except (xml.parsers.expat.ExpatError, IndexError) as e:
+            raise XmlResponseInvalidError(
+                "The XML response from the OSM API is invalid: %r" % e
+            )
+
         result = []
         for action in data.childNodes:
             if action.nodeName == "#text":
@@ -1790,11 +1823,9 @@ class OsmApi:
                 { ... }
             ]
         """
-        data = xml.dom.minidom.parseString(data)
+        noteElements = self._OsmResponseToDom(data, tag="note")
         result = []
-        osm_data = data.getElementsByTagName("osm")[0]
-
-        for noteElement in osm_data.getElementsByTagName("note"):
+        for noteElement in noteElements:
             note = self._DomParseNote(noteElement)
             result.append(note)
         return result
@@ -1885,7 +1916,7 @@ class OsmApi:
             self._changesetautocpt = 0
         return None
 
-    def _http_request(self, method, path, auth, send):  # noqa
+    def _http_request(self, method, path, auth, send, return_value=True):  # noqa
         """
         Returns the response generated by an HTTP request.
 
@@ -1898,6 +1929,17 @@ class OsmApi:
         be preformed on this request.
         `send` contains additional data that might be sent in a
         request.
+        `return_value` indicates wheter this request should return
+        any data or not.
+
+        If the username or password is missing,
+        `OsmApi.UsernamePasswordMissingError` is raised.
+
+        If the requested element has been deleted,
+        `OsmApi.ElementDeletedApiError` is raised.
+
+        If the response status code indicates an error,
+        `OsmApi.ApiError` is raised.
         """
         if self._debug:
             error_msg = (
@@ -1919,10 +1961,21 @@ class OsmApi:
         response = self._session.request(method, path, auth=user_pass,
                                          data=send)
         if response.status_code != 200:
-            if response.status_code == 410:
-                return None
             payload = response.content.strip()
+            if response.status_code == 410:
+                raise ElementDeletedApiError(
+                    response.status_code,
+                    response.reason,
+                    payload
+                )
             raise ApiError(response.status_code, response.reason, payload)
+        if return_value and not response.content:
+            raise ResponseEmptyApiError(
+                response.status_code,
+                response.reason,
+                ''
+            )
+
         if self._debug:
             error_msg = (
                 "%s %s %s"
@@ -1931,12 +1984,18 @@ class OsmApi:
             print(error_msg, file=sys.stderr)
         return response.content
 
-    def _http(self, cmd, path, auth, send):  # noqa
+    def _http(self, cmd, path, auth, send, return_value=True):  # noqa
         i = 0
         while True:
             i += 1
             try:
-                return self._http_request(cmd, path, auth, send)
+                return self._http_request(
+                    cmd,
+                    path,
+                    auth,
+                    send,
+                    return_value=return_value
+                )
             except ApiError as e:
                 if e.status >= 500:
                     if i == self.MAX_RETRY_LIMIT:
@@ -1974,8 +2033,8 @@ class OsmApi:
     def _get(self, path):
         return self._http('GET', path, False, None)
 
-    def _put(self, path, data):
-        return self._http('PUT', path, True, data)
+    def _put(self, path, data, return_value=True):
+        return self._http('PUT', path, True, data, return_value=return_value)
 
     def _post(self, path, data, optionalAuth=False):
         auth = True
@@ -1991,6 +2050,24 @@ class OsmApi:
     # Internal dom function                          #
     ##################################################
 
+    def _OsmResponseToDom(self, response, tag, single=False):
+        """
+        Returns the (sub-) DOM parsed from an OSM response
+        """
+        try:
+            dom = xml.dom.minidom.parseString(response)
+            osm_dom = dom.getElementsByTagName("osm")[0]
+            all_data = osm_dom.getElementsByTagName(tag)
+            first_element = all_data[0]
+        except (xml.parsers.expat.ExpatError, IndexError) as e:
+            raise XmlResponseInvalidError(
+                "The XML response from the OSM API is invalid: %r" % e
+            )
+
+        if single:
+            return first_element
+        return all_data
+
     def _DomGetAttributes(self, DomElement):  # noqa
         """
         Returns a formated dictionnary of attributes of a DomElement.
